/**
 * \addtogroup SD_Card
 * @{
 */

/*****************************************************************************/
/**
 * \file SD_Card.h
 * Sector level SD card routines for the 8051 and Keil C51.
 * \author Copyright (c) 2008, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.0
 * \date 27-11-08
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#ifndef SD_CARD_H
#define SD_CARD_H

#include "Reg51.h"
#include <intrins.h>

/*****************************************************************************
 *
 *                               Macro Definitions
 *
 *****************************************************************************/

/** 
 * SD card device docking status macro.
 * \param -
 * \return \b true once the SD card is fully inserted into it's receptacle.
 * \hideinitializer
 */
#define SD_CARD_DOCKED()  ~SD_CARD_CD


/** 
 * SD card device write protect status macro.
 * \param -
 * \return \b true if the SD card is write protected.
 * \hideinitializer
 */
#define SD_CARD_WRITE_PROTECTED()  ~SD_CARD_WP



/*****************************************************************************
 *
 *                               Defined Constants
 *
 *****************************************************************************/

/** 
 * Microcontroller to SD card communications mode for hardware or software SPI.
 * Set \b SD_CARD_HARDWARE_SPI to 1 if hardware SPI available, 0 otherwise. 
 * \hideinitializer
 */
#define SD_CARD_HARDWARE_SPI     0


/** 
 * Microcontroller port assigned to communicate with the SD card device.
 * Adjust \b SD_CARD_PORT to match hardware. 
 * \hideinitializer
 */
#define SD_CARD_PORT     P1


/** 
 * SD card sector size.
 * Adjust \b SD_CARD_SECTOR_SIZE to match the sector size of the SD device
 * (typically 512).
 * \hideinitializer
 */
#define SD_CARD_SECTOR_SIZE  512u


/*
 * SD card bit assignments.
 */
sbit SD_CARD_WP   = SD_CARD_PORT^0;    // SD card write protect input
sbit SD_CARD_SS   = SD_CARD_PORT^1;    // Defined but not used
sbit SD_CARD_CD   = SD_CARD_PORT^2;    // SD card detect input
sbit SD_CARD_CS   = SD_CARD_PORT^3;    // SD card select output
sbit SD_CARD_MISO = SD_CARD_PORT^5;    // SPI data in input
sbit SD_CARD_SCK  = SD_CARD_PORT^6;    // SPI serial clock output
sbit SD_CARD_MOSI = SD_CARD_PORT^7;    // SPI data out output


/*
 * Serial peripheral control register bit masks
 */
#define SPR2    0x80    // Serial peripheral rate 2
#define SPEN    0x40    // Serial peripheral enable
#define SSDIS   0x20    // SS bar disable
#define MSTR    0x10    // Serial peripheral master
#define CPOL    0x08    // Clock polarity
#define CPHA    0x04    // Clock phase
#define SPR1    0x20    // Serial peripheral rate 1
#define SPR0    0x10    // Serial peripheral rate 0


/*
 * Serial peripheral status and control register bit masks
 */
#define SPIF    0x80    // Serial peripheral data transfer flag
#define WCOL    0x40    // Write collision flag
#define SSERR   0x20    // Synchronous serial slave error flag
#define MODF    0x10    // Mode fault


/*
 * SD card command assignments.
 *
 * The following command numbers have been OR'ed with 0x40 to produce the 
 * 01xxxxxxb command start bit sequence.
 */
#define SD_CARD_CMD0        0x40    // GO_IDLE_STATE - Resets the SD memory
                                    // card.
#define SD_CARD_CMD0_CRC    0x95    // Pre-computed CRC.
#define SD_CARD_CMD55       0x77    // Next command is application specific.
#define SD_CARD_ACMD41      0x69    // SD_SEND_OP_COND - Sends host capacity
                                    // support information and activates the
                                    // card's initialisation process.
#define SD_CARD_CMD17       0x51    // READ_SINGLE_BLOCK - Reads a block of
                                    // the size selected by the SET_BLOCKLEN
                                    // command.
#define SD_CARD_CMD24       0x58    // WRITE_BLOCK - Writes a block of
                                    // the size selected by the SET_BLOCKLEN
                                    // command.
#define SD_CARD_CDM10       0x4A    // SEND_CID - Asks the selected card to 
                                    // send its card identification (CID).
#define SD_CARD_CDM9        0x49    // SEND_CSD - Asks the selected card to 
                                    // send its card-specific data (CSD).


/*
 * SPI control token assignments.
 */
#define SD_CARD_START_BLOCK_TOKEN  0xFE    // Marks the start of a data block
                                           // transfer.


/*
 * Function return values
 */
#define SD_CARD_NO_ERROR         0
#define SD_CARD_ERROR            1
#define SD_CARD_NOT_SD           2


/*
 * R1 response assignments
 */
#define SD_CARD_IN_IDLE_STATE    1
#define SD_CARD_NO_RESPONSE   0xFF



/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/

/** 
 * SD card module initialisation function.
 * Must be called once to set-up SD card hardware prior to calling any sector
 * level SD routines.
 * \param -
 * \return SD_CARD_ERROR, SD_CARD_NOT_SD, SD_CARD_NO_ERROR
 */
extern unsigned char sd_card_init(void);


/** 
 * SD card sector level read function.
 * Reads an SD sector, of length \b SD_CARD_SECTOR_SIZE and specified by the
 * given sector number, into a memory buffer referenced by the given pointer.
 * \param sector An \b unsigned \b long LBA sector number.
 * \param buffer An \b unsigned \b char \b pointer to \b xdata memory that
 *         will receive the read sector data.
 * \return -
 */
extern void sd_card_rd_sector(unsigned long sector,
                             unsigned char xdata * buffer);


/** 
 * SD card sector level write function.
 * Writes an SD sector, of length \b SD_CARD_SECTOR_SIZE and specified by the
 * given sector number, from a memory buffer referenced by the given pointer.
 * \param sector An \b unsigned \b long LBA sector number.
 * \param buffer An \b unsigned \b char \b pointer to \b xdata memory that
 *         holds the sector data to write.
 * \return -
 */
extern void sd_card_wr_sector(unsigned long sector,
                             unsigned char xdata * buffer);


/** 
 * SD card identification register (CID) retrieval function.
 * \param buffer An \b unsigned \b char \b pointer to \b xdata memory that
 *         receives the CID register contents.
 * \return -
 */
extern void sd_card_read_cid(unsigned char xdata * buffer);


/** 
 * SD card-specific data register (CSD) retrieval function.
 * \param buffer An \b unsigned \b char \b pointer to \b xdata memory that
 *         receives the CSD register contents.
 * \return -
 */
extern void sd_card_read_csd(unsigned char xdata * buffer);


#endif

/** @} */
